#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains base load widget for data items
"""

from __future__ import print_function, division, absolute_import

import os
import logging

from Qt.QtCore import QSize
from Qt.QtWidgets import QSizePolicy, QFrame, QMenu
from Qt.QtGui import QCursor

from tpDcc import dcc
from tpDcc.managers import resources
from tpDcc.libs.python import decorators
from tpDcc.libs.qt.core import base
from tpDcc.libs.qt.widgets import layouts, label, group, buttons, formwidget, dividers
from tpDcc.libs.datalibrary.core import version as data_version

from tpDcc.tools.datalibrary.data import base as base_data
from tpDcc.tools.datalibrary.widgets import sequence, version

LOGGER = logging.getLogger('tpDcc-libs-datalibrary')


class _MetaLoadWidget(type):

    def __call__(self, *args, **kwargs):
        as_class = kwargs.get('as_class', False)
        if dcc.client().is_maya():
            from tpDcc.tools.datalibrary.dccs.maya.widgets import load
            if as_class:
                return load.MayaLoadWidget
            else:
                return type.__call__(load.MayaLoadWidget, *args, **kwargs)
        else:
            if as_class:
                return BaseLoadWidget
            else:
                return type.__call__(BaseLoadWidget, *args, **kwargs)


class BaseLoadWidget(base.BaseWidget, object):
    def __init__(self, item_view, client=None, parent=None):

        self._item_view = item_view
        self._client = client
        self._form_widget = None
        self._sequence_widget = None

        super(BaseLoadWidget, self).__init__(parent=parent)

        self.setObjectName('LoadWidget')

        try:
            self.form_widget().validate()

        except NameError as error:
            LOGGER.exception(error)

        self.update_thumbnail_size()

        item = self.item()
        if item:
            self._accept_button.setVisible(bool(item.functionality().get('load', False)))
            self._update_version_info()

    # ============================================================================================================
    # OVERRIDES
    # ============================================================================================================

    def get_main_layout(self):
        return layouts.VerticalLayout(spacing=0, margins=(0, 0, 0, 0))

    def ui(self):
        super(BaseLoadWidget, self).ui()

        self.setWindowTitle('Load Item')

        title_frame = QFrame(self)
        title_frame_layout = layouts.VerticalLayout(spacing=0, margins=(0, 0, 0, 0))
        title_frame.setLayout(title_frame_layout)
        title_widget = QFrame(self)
        title_layout = layouts.VerticalLayout(spacing=0, margins=(0, 0, 0, 0))
        title_widget.setLayout(title_layout)
        title_buttons_layout = layouts.HorizontalLayout(spacing=0, margins=(0, 0, 0, 0))
        title_layout.addLayout(title_buttons_layout)
        title_icon = label.BaseLabel(parent=self)
        title_button = label.BaseLabel(self.item().label(), parent=self)
        title_button.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
        self._menu_button = buttons.BaseButton(parent=self)
        self._menu_button.setIcon(resources.icon('menu_dots'))
        title_buttons_layout.addWidget(title_icon)
        title_buttons_layout.addWidget(title_button)
        title_buttons_layout.addWidget(self._menu_button)
        title_frame_layout.addWidget(title_widget)

        item_icon_name = self.item().icon() or 'tpDcc'
        item_icon = resources.icon(item_icon_name)
        if not item_icon:
            item_icon = resources.icon('tpDcc')
        title_icon.setPixmap(item_icon.pixmap(QSize(20, 20)))

        main_frame = QFrame(self)
        main_frame_layout = layouts.VerticalLayout(spacing=0, margins=(0, 0, 0, 0))
        main_frame.setLayout(main_frame_layout)
        icon_frame = QFrame(self)
        icon_frame_layout = layouts.VerticalLayout(spacing=0, margins=(0, 0, 0, 0))
        icon_frame.setLayout(icon_frame_layout)
        icon_title_frame = QFrame(self)
        icon_title_frame_layout = layouts.VerticalLayout(spacing=0, margins=(0, 0, 0, 0))
        icon_frame_layout.addWidget(icon_title_frame)
        icon_title_frame.setLayout(icon_title_frame_layout)
        icon_frame2 = QFrame(self)
        icon_frame2_layout = layouts.VerticalLayout(spacing=0, margins=(0, 0, 0, 0))
        icon_frame2.setLayout(icon_frame2_layout)
        thumbnail_layout = layouts.HorizontalLayout(spacing=0, margins=(0, 0, 0, 0))
        self._thumbnail_frame = QFrame(self)
        thumbnail_frame_layout = layouts.VerticalLayout(spacing=0, margins=(0, 0, 0, 0))
        self._thumbnail_frame.setLayout(thumbnail_frame_layout)
        icon_frame_layout.addWidget(icon_frame2)
        icon_frame2_layout.addLayout(thumbnail_layout)
        thumbnail_layout.addWidget(self._thumbnail_frame)
        form_frame = QFrame(self)
        form_frame_layout = layouts.VerticalLayout(spacing=0, margins=(0, 0, 0, 0))
        form_frame.setLayout(form_frame_layout)
        main_frame_layout.addWidget(icon_frame)
        main_frame_layout.addWidget(form_frame)

        version_frame = QFrame(self)
        version_frame_layout = layouts.VerticalLayout(spacing=0, margins=(0, 0, 0, 0))
        version_frame.setLayout(version_frame_layout)
        self._versions_widget = version.VersionHistoryWidget(parent=self)
        version_frame_layout.addWidget(self._versions_widget)

        self._custom_widget_frame = QFrame(self)
        custom_widget_layout = layouts.VerticalLayout(spacing=0, margins=(0, 0, 0, 0))
        self._custom_widget_frame.setLayout(custom_widget_layout)

        self._preview_buttons_frame = QFrame(self)
        preview_buttons_layout = layouts.HorizontalLayout(spacing=2, margins=(0, 0, 0, 0))
        self._preview_buttons_frame.setLayout(preview_buttons_layout)
        self._accept_button = buttons.BaseButton('Load', parent=self)
        self._accept_button.setIcon(resources.icon('open'))
        preview_buttons_layout.addStretch()
        preview_buttons_layout.addWidget(self._accept_button)
        preview_buttons_layout.addStretch()

        self._export_btn = buttons.BaseButton('Export', parent=self)
        self._export_btn.setIcon(resources.icon('export'))
        self._import_btn = buttons.BaseButton('Import', parent=self)
        self._import_btn.setIcon(resources.icon('import'))
        self._reference_btn = buttons.BaseButton('Reference', parent=self)
        self._reference_btn.setIcon(resources.icon('reference'))
        for btn in [self._export_btn, self._import_btn, self._reference_btn]:
            btn.setToolTip(btn.text())
        extra_buttons_frame = QFrame(self)
        extra_buttons_layout = layouts.HorizontalLayout(spacing=2, margins=(0, 0, 0, 0))
        extra_buttons_frame.setLayout(extra_buttons_layout)
        extra_buttons_layout.addWidget(self._export_btn)
        extra_buttons_layout.addWidget(self._import_btn)
        extra_buttons_layout.addWidget(self._reference_btn)

        group_box = group.GroupBoxWidget('Icon', icon_frame)
        group_box.set_persistent(True)
        group_box.set_checked(True)

        self._version_box = group.GroupBoxWidget('Version', version_frame)
        self._version_box.set_persistent(True)
        self._version_box.set_checked(True)

        self._sequence_widget = sequence.ImageSequenceWidget(self)
        thumbnail_frame_layout.insertWidget(0, self._sequence_widget)

        if os.path.exists(self._item_view.image_sequence_path()):
            self._sequence_widget.set_path(self._item_view.image_sequence_path())
        elif os.path.exists(self._item_view.thumbnail_path()):
            self._sequence_widget.set_path(self._item_view.thumbnail_path())

        self._form_widget = formwidget.FormWidget(self)
        self._form_widget.set_schema(self.item().load_schema())
        self._form_widget.set_validator(self.item().load_validator)
        form_frame_layout.addWidget(self._form_widget)

        self.main_layout.addWidget(title_frame)
        self.main_layout.addWidget(group_box)
        self.main_layout.addWidget(main_frame)
        self.main_layout.addWidget(self._version_box)
        self.main_layout.addWidget(version_frame)
        self.main_layout.addWidget(self._custom_widget_frame)
        self.main_layout.addStretch()
        self.main_layout.addWidget(dividers.Divider())
        self.main_layout.addWidget(self._preview_buttons_frame)
        self.main_layout.addWidget(extra_buttons_frame)

    def setup_signals(self):
        self._menu_button.clicked.connect(self._on_show_menu)
        self._accept_button.clicked.connect(self._on_load)
        self._export_btn.clicked.connect(self._on_export)
        self._import_btn.clicked.connect(self._on_import)
        self._reference_btn.clicked.connect(self._on_reference)
        self._item_view.loadValueChanged.connect(self._on_item_value_changed)

    def resizeEvent(self, event):
        """
        Overrides base QWidget resizeEvent function
        :param event: QResizeEvent
        """

        self.update_thumbnail_size()

    def close(self):
        """
        Overrides base QWidget close function to disable script job when its is done
        """

        if self.form_widget():
            self.form_widget().save_persistent_values()

        super(BaseLoadWidget, self).close()

    # ============================================================================================================
    # BASE
    # ============================================================================================================

    def item(self):
        """
        Returns the library item to load
        :return:
        """

        return self._item_view.item

    def item_view(self):
        """
        Returns the library item view to load
        :return: LibraryItem
        """

        return self._item_view

    def form_widget(self):
        """
        Returns form widget instance
        :return: FormWidget
        """

        return self._form_widget

    def set_custom_widget(self, widget):
        """
        Sets custom widget to use when loading items
        :param widget: QQWidget
        """

        self._custom_widget_frame.layout().addWidget(widget)

    def update_thumbnail_size(self):
        """
        Updates the thumbnail button to teh size of the widget
        """

        width = self.width() - 10
        if width > 250:
            width = 250
        size = QSize(width, width)
        self._sequence_widget.setIconSize(size)
        self._sequence_widget.setMaximumSize(size)
        self._thumbnail_frame.setMaximumSize(size)

    # ============================================================================================================
    # INTERNAL
    # ============================================================================================================

    def _update_version_info(self):
        item_view = self.item_view()
        if not item_view or not item_view.library_window():
            return

        library_window = item_view.library_window()
        if not library_window:
            return

        repository_type = library_window.get_repository_type()
        repository_path = library_window.get_repository_path()
        if not repository_type or not repository_path:
            return

        try:
            repository_type = int(repository_type)
        except Exception:
            repository_type = 0

        if repository_type == 0:
            repository_type = None
        elif repository_type == 1:
            repository_type = data_version.GitVersionControl

        self._versions_widget.set_version_control_class(repository_type)
        self._versions_widget.set_repository_path(repository_path)

        # If not valid version control is defined, we hide version control
        valid_version_control = self._versions_widget.set_directory(self.item().format_identifier())
        self._versions_widget.setVisible(valid_version_control)
        self._version_box.setVisible(valid_version_control)

    # ============================================================================================================
    # CALLBACKS
    # ============================================================================================================

    def _on_show_menu(self):
        """
        Internal callback function that is called when menu button is clicked byu the user
        :return: QAction
        """

        menu = QMenu(self)
        self._item_view.context_edit_menu(menu)
        point = QCursor.pos()
        point.setX(point.x() + 3)
        point.setY(point.y() + 3)

        return menu.exec_(point)

    def _on_load(self):
        """
        Internal callback function that is called when Load button is pressed by the user
        """

        load_function = self.item().functionality().get('load')
        if not load_function:
            LOGGER.warning('Load functionality is not available for data: "{}"'.format(self.item()))
            return

        library_path = self.item().library.identifier
        if not library_path or not os.path.isfile(library_path):
            LOGGER.warning('Impossible to load data "{}" because its library does not exists: "{}"'.format(
                self.item(), library_path))
            return

        if self._client:
            self._client().load_data(library_path=library_path, data_path=self.item().format_identifier())
        else:
            load_function()

    def _on_export(self):
        """
        Internal callback function that is called when export button is pressed by the user
        """

        item = self.item()
        if not item:
            return

        library_window = self.item_view().library_window()
        if not library_window:
            return

        base_data.BaseDataItemView.show_export_widget(item.__class__, item.format_identifier(), library_window)

    def _on_import(self):
        """
        Internal callback function that is called when import button is pressed by the user
        """

        import_function = self.item().functionality().get('import_data')
        if not import_function:
            LOGGER.warning('Import functionality is not available for data: "{}"'.format(self.item()))
            return

        library_path = self.item().library.identifier
        if not library_path or not os.path.isfile(library_path):
            LOGGER.warning('Impossible to load data "{}" because its library does not exists: "{}"'.format(
                self.item(), library_path))
            return

        if self._client:
            self._client().import_data(library_path=library_path, data_path=self.item().format_identifier())
        else:
            import_function()

    def _on_reference(self):
        """
        Internal callback function that is called when reference button is pressed by the user
        """

        reference_function = self.item().functionality().get('reference_data')
        if not reference_function:
            LOGGER.warning('Reference functionality is not available for data: "{}"'.format(self.item()))
            return

        library_path = self.item().library.identifier
        if not library_path or not os.path.isfile(library_path):
            LOGGER.warning('Impossible to load data "{}" because its library does not exists: "{}"'.format(
                self.item(), library_path))
            return

        if self._client:
            self._client().reference_data(library_path=library_path, data_path=self.item().format_identifier())
        else:
            reference_function()

    def _on_item_value_changed(self, field, value):
        """
        Internal callback function that is called each time an item field value changes
        :param field: str
        :param value: object
        """

        self._form_widget.set_value(field, value)


@decorators.add_metaclass(_MetaLoadWidget)
class LoadWidget(object):
    pass



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains base save widget for data items
"""

from __future__ import print_function, division, absolute_import

import os
import logging
import traceback

from Qt.QtCore import Signal, QSize
from Qt.QtWidgets import QSizePolicy, QFrame, QDialogButtonBox, QFileDialog

from tpDcc import dcc
from tpDcc.managers import resources
from tpDcc.libs.resources.core import theme
from tpDcc.libs.python import decorators
from tpDcc.libs.qt.core import base, qtutils
from tpDcc.libs.qt.widgets import layouts, label, buttons, formwidget, messagebox, snapshot

from tpDcc.tools.datalibrary.core import utils
from tpDcc.tools.datalibrary.widgets import sequence

LOGGER = logging.getLogger('tpDcc-libs-datalibrary')


class _MetaSaveWidget(type):

    def __call__(self, *args, **kwargs):
        as_class = kwargs.get('as_class', False)
        if dcc.client().is_maya():
            from tpDcc.tools.datalibrary.dccs.maya.widgets import save
            if as_class:
                return save.MayaSaveWidget
            else:
                return type.__call__(save.MayaSaveWidget, *args, **kwargs)
        else:
            if as_class:
                return BaseSaveWidget
            else:
                return type.__call__(BaseSaveWidget, *args, **kwargs)


@theme.mixin
class BaseSaveWidget(base.BaseWidget, object):

    cancelled = Signal()
    saved = Signal()

    ENABLE_THUMBNAIL_CAPTURE = True

    def __init__(self, item_view, client=None, *args, **kwargs):

        self._item_view = item_view
        self._client = client
        self._form_widget = None
        self._sequence_widget = None

        super(BaseSaveWidget, self).__init__(*args, **kwargs)

        self.setObjectName('LibrarySaveWidget')

        self._create_sequence_widget()
        self.update_thumbnail_size()
        self.set_item_view(item_view)

    # ============================================================================================================
    # OVERRIDES
    # ============================================================================================================

    def get_main_layout(self):
        return layouts.VerticalLayout(spacing=4, margins=(0, 0, 0, 0))

    def ui(self):
        super(BaseSaveWidget, self).ui()

        self.setWindowTitle('Save Item')

        title_frame = QFrame(self)
        title_frame_layout = layouts.VerticalLayout(spacing=0, margins=(0, 0, 0, 0))
        title_frame.setLayout(title_frame_layout)
        title_widget = QFrame(self)
        title_layout = layouts.VerticalLayout(spacing=0, margins=(0, 0, 0, 0))
        title_widget.setLayout(title_layout)
        title_buttons_layout = layouts.HorizontalLayout(spacing=0, margins=(0, 0, 0, 0))
        title_layout.addLayout(title_buttons_layout)
        title_icon = label.BaseLabel(parent=self)
        title_button = label.BaseLabel(self.item().menu_name(), parent=self)
        title_button.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
        self._menu_button = buttons.BaseButton(parent=self)
        self._menu_button.setIcon(resources.icon('menu_dots'))
        self._menu_button.setVisible(False)     # Hide by default
        title_buttons_layout.addWidget(title_icon)
        title_buttons_layout.addSpacing(5)
        title_buttons_layout.addWidget(title_button)
        title_buttons_layout.addWidget(self._menu_button)
        title_frame_layout.addWidget(title_widget)

        item_icon_name = self.item().icon() or 'tpDcc'
        item_icon = resources.icon(item_icon_name)
        if not item_icon:
            item_icon = resources.icon('tpDcc')
        title_icon.setPixmap(item_icon.pixmap(QSize(20, 20)))

        thumbnail_layout = layouts.HorizontalLayout(spacing=0, margins=(0, 0, 0, 0))
        self._thumbnail_frame = QFrame(self)
        thumbnail_frame_layout = layouts.VerticalLayout(spacing=0, margins=(0, 2, 0, 2))
        self._thumbnail_frame.setLayout(thumbnail_frame_layout)
        thumbnail_layout.addWidget(self._thumbnail_frame)

        self._options_frame = QFrame(self)
        options_frame_layout = layouts.VerticalLayout(spacing=0, margins=(4, 2, 4, 2))
        self._options_frame.setLayout(options_frame_layout)

        preview_buttons_frame = QFrame(self)
        self._preview_buttons_layout = layouts.HorizontalLayout(spacing=0, margins=(4, 2, 4, 2))
        preview_buttons_frame.setLayout(self._preview_buttons_layout)
        self._save_button = buttons.BaseButton('Save', parent=self)
        self._save_button.setIcon(resources.icon('save'))
        self._cancel_button = buttons.BaseButton('Cancel', parent=self)
        self._cancel_button.setIcon(resources.icon('cancel'))
        self._preview_buttons_layout.addStretch()
        self._preview_buttons_layout.addWidget(self._save_button)
        self._preview_buttons_layout.addStretch()
        self._preview_buttons_layout.addWidget(self._cancel_button)
        self._preview_buttons_layout.addStretch()

        self.main_layout.addWidget(title_frame)
        self.main_layout.addLayout(thumbnail_layout)
        self.main_layout.addWidget(self._options_frame)
        self.main_layout.addWidget(preview_buttons_frame)

    def setup_signals(self):
        self._menu_button.clicked.connect(self._on_show_menu)
        self._save_button.clicked.connect(self._on_save)
        self._cancel_button.clicked.connect(self._on_cancel)

    def resizeEvent(self, event):
        """
        Overrides base QWidget resizeEvent function
        :param event: QResizeEvent
        """

        self.update_thumbnail_size()

    def close(self):
        """
        Overrides base QWidget close function to disable script job when its is done
        """

        if self._form_widget:
            self._form_widget.save_persistent_values()

        super(BaseSaveWidget, self).close()

    # ============================================================================================================
    # BASE
    # ============================================================================================================

    def folder_path(self):
        """
        Returns the folder path
        :return: str
        """

        return self.form_widget().value('folder')

    def set_folder_path(self, path):
        """
        Sets the destination folder path
        :param path: str
        """

        self.form_widget().set_value('folder', path)

    def set_thumbnail_path(self, path):
        """
        Sets the path to the thumbnail image or the image sequence directory
        :param path: str
        """

        file_name, extension = os.path.splitext(path)
        target = utils.temp_path('thumbnail{}'.format(extension))
        utils.copy_path(path, target, force=True)

        self._sequence_widget.set_path(target)

    def library_window(self):
        """
        Returns library widget window for the item
        :return: LibraryWindow
        """

        return self.item_view().library_window()

    def set_library_window(self, library_window):
        """
        Sets the library widget for the item
        :param library_window: LibraryWindow
        """

        self.item_view().set_library_window(library_window)

    def form_widget(self):
        """
        Returns the form widget instance
        :return: FormWidget
        """

        return self._form_widget

    def item(self):
        """
        Returns current item
        :return:
        """

        return self.item_view().item

    def item_view(self):
        """
        Returns the current item view
        :return: LibraryItem
        """

        return self._item_view

    def set_item_view(self, item_view):
        """
        Sets the base item to be created
        :param item_view: LibraryItem
        """

        self._item_view = item_view

        if os.path.exists(item_view.image_sequence_path()):
            self.set_thumbnail_path(item_view.image_sequence_path())
        elif not item_view.is_default_thumbnail_path():
            self.set_thumbnail_path(item_view.thumbnail_path())

        schema = self.item().save_schema()
        if schema:
            form_widget = formwidget.FormWidget(self)
            form_widget.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
            form_widget.set_schema(schema)
            form_widget.set_validator(self.item().save_validator)
            # item_name = os.path.basename(item.path())
            # form_widget.set_values({'name': item_name})
            self._options_frame.layout().addWidget(form_widget)
            form_widget.validate()
            self._form_widget = form_widget
        else:
            self._options_frame.setVisible(False)

    def update_thumbnail_size(self):
        """
        Updates the thumbnail button to teh size of the widget
        """

        width = self.width() - 10
        if width > 250:
            width = 250
        size = QSize(width, width)
        if self._sequence_widget:
            self._sequence_widget.setIconSize(size)
            self._sequence_widget.setMaximumSize(size)
        self._thumbnail_frame.setMaximumSize(size)

    def show_thumbnail_capture_dialog(self):
        """
        Asks the user if they would like to capture a thumbnail
        :return: int
        """

        buttons = QDialogButtonBox.Yes | QDialogButtonBox.Ignore | QDialogButtonBox.Cancel
        parent = self.item_view().library_window()
        btn = messagebox.MessageBox.question(
            parent, 'Create a thumbnail', 'Would you like to capture a thumbnail?', buttons=buttons)
        if btn == QDialogButtonBox.Yes:
            self.thumbnail_capture()

        return btn

    def show_by_frame_dialog(self):
        """
        Show the by frame dialog
        """

        help_text = """
        To help speed up the playblast you can set the "by frame" to another greater than 1.
        For example if the "by frame" is set to 2 it will playblast every second frame
        """

        result = None
        options = self.form_widget().values()
        by_frame = options.get('byFrame', 1)
        start_frame, end_frame = options.get('frameRange', [None, None])

        duration = end_frame - start_frame if start_frame is not None and end_frame is not None else 1
        if duration > 100 and by_frame == 1:
            buttons = QDialogButtonBox.Ok | QDialogButtonBox.Cancel
            result = messagebox.MessageBox.question(
                self.library_window(), title='Tip', text=help_text, buttons=buttons, enable_dont_show_checkbox=True
            )

        return result

    def thumbnail_capture(self, show=False):
        """
        Captures a playblast and saves it to the temporal thumbnail path
        :param show: bool
        """

        options = self.form_widget().values()
        start_frame, end_frame = options.get('frameRange', [None, None])
        step = options.get('byFrame', 1)

        if not qtutils.is_control_modifier():
            result = self.show_by_frame_dialog()
            if result == QDialogButtonBox.Cancel:
                return

        path = utils.temp_path('sequence', 'thumbnail.jpg')

        try:
            snapshot.SnapshotWindow(path=path, on_save=self._on_thumbnail_captured)
            # thumbnail.ThumbnailCaptureDialog.thumbnail_capture(
            #     path=self._temp_path,
            #     show=show,
            #     start_frame=start_frame,
            #     end_frame=end_frame,
            #     step=step,
            #     clear_cache=False,
            #     captured=self._on_thumbnail_captured
            # )
        except Exception as e:
            messagebox.MessageBox.critical(self.library_window(), 'Error while capturing thumbnail', str(e))
            LOGGER.error(traceback.format_exc())

    def save(self, path, thumbnail):
        """
        Saves the item with the given objects to the given disk location path
        :param path: str
        :param thumbnail: str
        """

        kwargs = self.form_widget().values()
        sequence_path = self._sequence_widget.dirname()
        item_view = self.item_view()
        item_view.item_view.path = path
        library_window = self.library_window()
        valid_save = item_view.safe_save(thumbnail=thumbnail, sequence_path=sequence_path, **kwargs)
        if valid_save:
            if library_window:
                library_window.refresh()
                library_window.select_folder_path(path)
            self.saved.emit()
        self.close()

    # ============================================================================================================
    # INTERNAL
    # ============================================================================================================

    def _create_sequence_widget(self):
        """
        Internal function that creates a sequence widget to replace the static thumbnail widget
        """

        self._sequence_widget = sequence.ImageSequenceWidget(self)
        self._sequence_widget.setObjectName('thumbnailButton')
        self._thumbnail_frame.layout().insertWidget(0, self._sequence_widget)
        self._sequence_widget.clicked.connect(self._on_thumbnail_capture)
        self._sequence_widget.setToolTip(
            'Click to capture a thumbnail from the current model panel.\n'
            'CTRL + Click to show the capture window for better framing.')

        camera_icon = resources.get('icons', self.theme().style(), 'camera.png')
        expand_icon = resources.get('icons', self.theme().style(), 'full_screen.png')
        folder_icon = resources.get('icons', self.theme().style(), 'folder.png')

        self._sequence_widget.addAction(
            camera_icon, 'Capture new image', 'Capture new image', self._on_thumbnail_capture)
        self._sequence_widget.addAction(
            expand_icon, 'Show Capture window', 'Show Capture window', self._on_show_capture_window)
        self._sequence_widget.addAction(
            folder_icon, 'Load image from disk', 'Load image from disk', self._on_show_browse_image_dialog)

        self._sequence_widget.setIcon(resources.icon('tpdcc'))

    # ============================================================================================================
    # CALLBACKS
    # ============================================================================================================

    def _on_show_menu(self):
        """
        Internal callback function that is called when menu button is clicked byu the user
        :return: QAction
        """

        pass

    def _on_save(self):
        if not self.library_window():
            return False

        library = self.library_window().library()
        if not library:
            return False

        try:
            self.form_widget().validate()
            if self.form_widget().has_errors():
                raise Exception('\n'.join(self.form_widget().errors()))
            has_frames = self._sequence_widget.has_frames()
            if not has_frames and self.ENABLE_THUMBNAIL_CAPTURE:
                button = self.show_thumbnail_capture_dialog()
                if button == QDialogButtonBox.Cancel:
                    return False
            name = self.form_widget().value('name')
            folder = self.form_widget().value('folder')
            comment = self.form_widget().value('comment') or ''

            extension = self.item().extension()
            if extension and not name.endswith(extension):
                name = '{}{}'.format(name, extension)

            path = folder + '/' + name
            thumbnail = self._sequence_widget.first_frame()

            save_item = library.get(path, only_extension=True)
            save_function = save_item.functionality().get('save')
            if not save_function:
                LOGGER.warning('Item "{}" does not supports save operation'.format(save_item))
                return False

            library_path = self.item().library.identifier
            if not library_path or not os.path.isfile(library_path):
                LOGGER.warning('Impossible to save data "{}" because its library does not exists: "{}"'.format(
                    self.item(), library_path))
                return

            values = self.form_widget().values()
            try:
                if self._client:
                    success, message, dependencies = self._client().save_data(
                        library_path=library_path, data_path=path, values=values)
                    if not success:
                        messagebox.MessageBox.critical(self.library_window(), 'Error while saving', str(message))
                        LOGGER.error(str(message))
                        return False
                else:
                    dependencies = save_function(**values)
            except Exception as exc:
                messagebox.MessageBox.critical(self.library_window(), 'Error while saving', str(exc))
                LOGGER.error(traceback.format_exc())
                return False

        except Exception as exc:
            messagebox.MessageBox.critical(self.library_window(), 'Error while saving', str(exc))
            LOGGER.error(traceback.format_exc())
            raise

        new_item_path = save_item.format_identifier()
        if not new_item_path or not os.path.isfile(new_item_path):
            LOGGER.warning('Although saving process for item "{}" was completed, '
                           'it seems no new data has been generated!'.format(save_item))
            self.saved.emit()
            return False

        save_item.library.add(new_item_path)

        # # TODO: Instead of creating a local version, we will use a git system to upload our data to our project repo
        # # TODO: Should we save new versions of dependencies too?
        # valid = save_item.create_version(comment=comment)
        # if not valid:
        #     LOGGER.warning('Impossible to store new version for data "{}"'.format(save_item))

        if thumbnail and os.path.isfile(thumbnail):
            save_item.store_thumbnail(thumbnail)

        self.library_window().sync()

        save_item.update_dependencies(dependencies=dependencies)

        self.saved.emit()

        return True

    def _on_cancel(self):
        self.cancelled.emit()
        self.close()

    def _on_thumbnail_capture(self):
        """
        Internal callback function that is called when a thumbnail capture must be done
        """

        self.thumbnail_capture(show=False)

    def _on_thumbnail_captured(self, captured_path):
        """
        Internal callback function that is called when thumbnail is captured
        :param captured_path: str
        """

        thumb_path = os.path.dirname(captured_path)
        self.set_thumbnail_path(thumb_path)

    def _on_show_capture_window(self):
        """
        Internal callback function that shows the capture window for framing
        """

        self.thumbnail_capture(show=True)

    def _on_show_browse_image_dialog(self):
        """
        Internal callback function that shows a file dialog for choosing an image from disk
        """

        file_dialog = QFileDialog(self, caption='Open Image', filter='Image Files (*.png *.jpg)')
        file_dialog.fileSelected.connect(self.set_thumbnail_path)
        file_dialog.exec_()


@decorators.add_metaclass(_MetaSaveWidget)
class SaveWidget(object):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains tpDcc-tools-datalibrary client implementation
"""

import os

from tpDcc.core import client
from tpDcc.libs.python import path as path_utils
import tpDcc.libs.datalibrary


class DataLibraryClient(client.DccClient, object):

    PORT = 28231

    # =================================================================================================================
    # OVERRIDES
    # =================================================================================================================

    def _get_paths_to_update(self):
        paths_to_update = super(DataLibraryClient, self)._get_paths_to_update()

        paths_to_update['tpDcc.libs.datalibrary'] = path_utils.clean_path(
            os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(tpDcc.libs.datalibrary.__file__)))))

        return paths_to_update

    # =================================================================================================================
    # BASE
    # =================================================================================================================

    def load_data_items(self):
        cmd = {
            'cmd': 'load_data_items'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['result']

    def list_namespaces(self):
        """
        Returns a list of all available namespaces
        :return: list(str)
        """

        cmd = {
            'cmd': 'list_namespaces'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['result']

    def list_nodes(self, node_name=None, node_type=None, full_path=True):
        """
        Returns list of nodes with given types. If no type, all scene nodes will be listed
        :param node_name:
        :param node_type:
        :param full_path:
        :return: list(str)
        """

        cmd = {
            'cmd': 'list_nodes',
            'node_name': node_name,
            'node_type': node_type,
            'full_path': full_path
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['result']

    def set_focus(self, ui_name):
        """
        Sets the focus in the given UI element
        :param ui_name: str
        """

        cmd = {
            'cmd': 'set_focus',
            'ui_name': ui_name
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def save_data(self, library_path, data_path, values=None):
        """
        Saves given data in given library
        :param library_path: str
        :param data_path: str
        :param values: values, dict or None
        :return:
        """

        cmd = {
            'cmd': 'save_data',
            'library_path': library_path,
            'data_path': data_path,
            'values': values or dict()
        }

        reply_dict = self.send(cmd)

        return reply_dict['success'], reply_dict.get('msg', ''), reply_dict['result']

    def export_data(self, library_path, data_path, values=None):
        """
        Exports given data in given library
        :param library_path: str
        :param data_path: str
        :param values: values, dict or None
        :return:
        """

        cmd = {
            'cmd': 'export_data',
            'library_path': library_path,
            'data_path': data_path,
            'values': values or dict()
        }

        reply_dict = self.send(cmd)

        return reply_dict['success'], reply_dict.get('msg', ''), reply_dict['result']

    def load_data(self, library_path, data_path):
        """
        Loads given path in given library
        :param library_path: str
        :param data_path: str
        :return:
        """

        cmd = {
            'cmd': 'load_data',
            'library_path': library_path,
            'data_path': data_path
        }

        reply_dict = self.send(cmd)

        return reply_dict['success'], reply_dict.get('msg', '')

    def import_data(self, library_path, data_path):
        """
        Imports given path in given library
        :param library_path: str
        :param data_path: str
        :return:
        """

        cmd = {
            'cmd': 'import_data',
            'library_path': library_path,
            'data_path': data_path
        }

        reply_dict = self.send(cmd)

        return reply_dict['success'], reply_dict.get('msg', '')

    def reference_data(self, library_path, data_path):
        """
        References given path in given library
        :param library_path: str
        :param data_path: str
        :return:
        """

        cmd = {
            'cmd': 'reference_data',
            'library_path': library_path,
            'data_path': data_path
        }

        reply_dict = self.send(cmd)

        return reply_dict['success'], reply_dict.get('msg', '')



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains base export widget for data items
"""

from __future__ import print_function, division, absolute_import

import os
import logging
import traceback

from Qt.QtWidgets import QSizePolicy

from tpDcc.managers import resources
from tpDcc.libs.python import decorators
from tpDcc.libs.qt.widgets import layouts, buttons, formwidget, messagebox

from tpDcc.tools.datalibrary.widgets import save

LOGGER = logging.getLogger('tpDcc-libs-datalibrary')


class _MetaExportWidget(type):
    def __call__(self, *args, **kwargs):
        as_class = kwargs.get('as_class', False)
        if as_class:
            return BaseExportWidget
        else:
            return type.__call__(BaseExportWidget, *args, **kwargs)


class BaseExportWidget(save.SaveWidget(as_class=True)):

    ENABLE_THUMBNAIL_CAPTURE = False

    # ============================================================================================================
    # OVERRIDES
    # ============================================================================================================

    def get_main_layout(self):
        return layouts.VerticalLayout(spacing=4, margins=(0, 0, 0, 0))

    def ui(self):
        super(BaseExportWidget, self).ui()

        self.setWindowTitle('Export Item')

        self._export_button = buttons.BaseButton('Export', parent=self)
        self._export_button.setIcon(resources.icon('export'))
        self._preview_buttons_layout.insertWidget(1, self._export_button)

        self._save_button.setVisible(False)
        self._thumbnail_frame.setVisible(False)

    def setup_signals(self):
        super(BaseExportWidget, self).setup_signals()

        self._export_button.clicked.connect(self._on_export)

    # ============================================================================================================
    # OVERRIDES
    # ============================================================================================================

    def set_item_view(self, item_view):
        """
        Sets the base item to be created
        :param item_view: LibraryItem
        """

        self._item_view = item_view

        schema = self.item().export_schema()
        if schema:
            form_widget = formwidget.FormWidget(self)
            form_widget.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
            form_widget.set_schema(schema)
            form_widget.set_validator(self.item().save_validator)
            item_name = os.path.basename(self.item().format_identifier())
            item_folder = os.path.dirname(self.item().format_identifier())
            form_widget.set_values({'name': item_name, 'folder': item_folder})
            self._options_frame.layout().addWidget(form_widget)
            form_widget.validate()
            self._form_widget = form_widget
        else:
            self._options_frame.setVisible(False)

    # ============================================================================================================
    # CALLBACKS
    # ============================================================================================================

    def _on_export(self):
        if not self.library_window():
            return False

        library = self.library_window().library()
        if not library:
            return False

        try:
            self.form_widget().validate()
            if self.form_widget().has_errors():
                raise Exception('\n'.join(self.form_widget().errors()))
            name = self.form_widget().value('name')
            folder = self.form_widget().value('folder')
            comment = self.form_widget().value('comment') or ''

            extension = self.item().extension()
            if extension and not name.endswith(extension):
                name = '{}{}'.format(name, extension)

            path = folder + '/' + name

            export_item = library.get(path, only_extension=True)
            export_function = export_item.functionality().get('export_data')
            if not export_function:
                LOGGER.warning('Item "{}" does not supports export operation'.format(export_item))
                return False

            library_path = self.item().library.identifier
            if not library_path or not os.path.isfile(library_path):
                LOGGER.warning('Impossible to save data "{}" because its library does not exists: "{}"'.format(
                    self.item(), library_path))
                return

            values = self.form_widget().values()
            try:
                if self._client:
                    _, _, dependencies = self._client().export_data(
                        library_path=library_path, data_path=path, values=values)
                else:
                    dependencies = export_function(**values)
            except Exception as exc:
                messagebox.MessageBox.critical(self.library_window(), 'Error while exporting', str(exc))
                LOGGER.error(traceback.format_exc())
                return False

        except Exception as exc:
            messagebox.MessageBox.critical(self.library_window(), 'Error while exporting', str(exc))
            LOGGER.error(traceback.format_exc())
            raise

        item_path = export_item.format_identifier()
        if not item_path or not os.path.isfile(item_path):
            LOGGER.warning('Although exporting process for item "{}" was completed, '
                           'it seems data was not exported successfully!'.format(export_item))
            self.saved.emit()
            return False

        # # TODO: Instead of creating a local version, we will use a git system to upload our data to our project repo
        # valid = export_item.create_version(comment=comment)
        # if not valid:
        #     LOGGER.warning('Impossible to store new version for data "{}"'.format(export_item))

        self.library_window().sync()

        if dependencies:
            export_item.update_dependencies(dependencies=dependencies)

        self.saved.emit()

        return True


@decorators.add_metaclass(_MetaExportWidget)
class ExportWidget(object):
    pass



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains widgets related with data functionality
"""

from __future__ import print_function, division, absolute_import

from tpDcc.managers import resources
from tpDcc.libs.qt.core import preferences
from tpDcc.libs.qt.widgets import label, layouts, combobox


class DataRepositoryWidget(preferences.CategoryWidgetBase, object):

    CATEGORY = 'Repository'

    def __init__(self, parent=None):
        super(DataRepositoryWidget, self).__init__(parent=parent)

        self._fill_version_types_combo()

    def ui(self):
        super(DataRepositoryWidget, self).ui()

        version_control_layout = layouts.HorizontalLayout()
        version_control_label = label.BaseLabel('Version Control', parent=None)
        self._version_type_combo = combobox.BaseComboBox(parent=self)
        version_control_layout.addWidget(version_control_label)
        version_control_layout.addWidget(self._version_type_combo)

        self.main_layout.addLayout(version_control_layout)
        self.main_layout.addStretch()

    def init_defaults(self, settings):
        if not settings:
            return

        settings.set('version_control', 0, setting_group='Version')

    def show_widget(self, settings):
        if not settings:
            return

        version_control = settings.get('version_control', default_value=0, setting_group='Version')

        self._version_type_combo.setCurrentIndex(int(version_control))

    def serialize(self, settings):
        if not settings:
            return

        settings.set('version_control', self._version_type_combo.currentIndex(), setting_group='Version')

    def _fill_version_types_combo(self):
        """
        Internal callback function that fills with the different types of supported version controls
        """

        self._version_type_combo.clear()
        for version_type in ['none', 'git']:
            self._version_type_combo.addItem(resources.icon(version_type), version_type)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains different widgets used in libraries
"""

from __future__ import print_function, division, absolute_import

from Qt.QtCore import Qt, QSize
from Qt.QtWidgets import QSizePolicy, QWidget, QToolButton, QToolBar, QAction
from Qt.QtGui import QColor, QPainter, QBrush

from tpDcc.libs.resources.core import icon
from tpDcc.libs.qt.core import qtutils, image, animation


class ImageSequenceWidget(QToolButton, object):

    DEFAULT_PLAYHEAD_COLOR = QColor(255, 255, 255, 220)
    DEFAULT_PLAYHEAD_HEIGHT = 4

    def __init__(self, *args):
        super(ImageSequenceWidget, self).__init__(*args)

        self.setMouseTracking(True)

        self._image_sequence = image.ImageSequence('')
        self._image_sequence.frameChanged.connect(self._on_frame_changed)

        self._toolbar = QToolBar(self)
        animation.fade_out_widget(self._toolbar, duration=0)

        spacer = QWidget()
        spacer.setMaximumWidth(4)
        spacer.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
        self._toolbar.addWidget(spacer)

        spacer1 = QWidget()
        spacer1.setMaximumWidth(4)
        spacer1.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
        self._first_spacer = self._toolbar.addWidget(spacer1)

        self.set_size(150, 150)

    # =================================================================================================================
    # OVERRIDES
    # =================================================================================================================

    def addAction(self, path, text, tip, callback):
        """
        Overrides base QToolButton addAction function
        Adds aan action to the toolbar
        :param path: str
        :param text: str
        :param tip: str
        :param callback: fn
        :return: QAction
        """

        action_icon = icon.Icon.state_icon(
            path,
            color='rgb(250,250,250,160)',
            color_active='rgb(250,250,250,250)',
            color_disabled='rgb(0,0,0,20)'
        )
        action = QAction(action_icon, text, self._toolbar)
        action.setToolTip(tip)
        self._toolbar.insertAction(self._first_spacer, action)
        action.triggered.connect(callback)

        return action

    def actions(self):
        """
        Overrides base QToolButton actions function
        Returns all the actions that are a child of the toolbar
        :return: list(QAction)
        """

        actions = list()
        for child in self._toolbar.children():
            if isinstance(child, QAction):
                actions.append(child)

        return actions

    def resizeEvent(self, event):
        """
        Overrides base QToolButton resizeEvent function
        Called when the widget is resized
        :param event: QResizeEvent
        """

        self.update_toolbar()

    def enterEvent(self, event):
        """
        Overrides base QToolButton enterEvent function
        Starts playing the image sequence when the mouse enters the widget
        :param event: QEvent
        """

        self._image_sequence.start()
        animation.fade_in_widget(self._toolbar, duration=300)

    def leaveEvent(self, event):
        """
        Overrides base QToolButton leaveEvent function
        Stops playing the image sequence when the mouse leaves the widget
        :param event: QEvent
        """

        self._image_sequence.pause()
        animation.fade_out_widget(self._toolbar, duration=300)

    def mouseMoveEvent(self, event):
        """
        Overrides base QToolButton mouseMoveEvent function
        :param event: QEvent
        """

        if qtutils.is_control_modifier() and self._image_sequence.frame_count() > 1:
            percent = 1.0 - (float(self.width() - event.pos().x()) / float(self.width()))
            frame = int(self._image_sequence.frame_count() * percent)
            self._image_sequence.jump_to_frame(frame)
            frame_icon = self._image_sequence.current_icon()
            self.setIcon(frame_icon)

    def paintEvent(self, event):
        """
        Overrides base QToolButton paintEvent function
        Triggered on frame changed
        :param event: QEvent
        """

        super(ImageSequenceWidget, self).paintEvent(event)

        painter = QPainter()
        painter.begin(self)
        if self.current_filename() and self._image_sequence.frame_count() > 1:
            r = event.rect()
            playhead_height = self.playhead_height()
            playhead_pos = self._image_sequence.percent() * r.width() - 1
            x = r.x()
            y = self.height() - playhead_height
            painter.seten(Qt.NoPen)
            painter.setBrush(QBrush(self.DEFAULT_PLAYHEAD_COLOR))
            painter.drawRect(x, y, playhead_pos, playhead_height)

        painter.end()

    # =================================================================================================================
    # BASE
    # =================================================================================================================

    def is_sequence(self):
        """
        Returns whether or not the image sequence has more than one frame
        :return: bool
        """

        return bool(self._image_sequence.frame_count() > 1)

    def dirname(self):
        """
        Returns the directory where the image sequence is located on disk
        :return: str
        """

        return self._image_sequence.dirname()

    def has_frames(self):
        """
        Returns whether or not image sequences has any frames
        :return: bool
        """

        return bool(self.first_frame())

    def first_frame(self):
        """
        Returns first frame path in the image sequence
        :return: str
        """

        return self._image_sequence.first_frame()

    def set_path(self, path):
        """
        Sets a single frame image sequence
        :param path: str
        """

        self._image_sequence.set_path(path)
        self.update_icon()

    def set_dirname(self, dirname):
        """
        Sets the location of the image sequence
        :param dirname: str
        """

        self._image_sequence.set_dirname(dirname)
        self.update_icon()

    def current_filename(self):
        """
        Returns the current image location
        :return: str
        """

        return self._image_sequence.current_filename()

    def playhead_height(self):
        """
        Returns the height of the playhead
        :return: int
        """

        return self.DEFAULT_PLAYHEAD_COLOR

    def set_size(self, w, h):
        """
        Set the size of the widget and updates icon size at the same time
        :param w: int
        :param h: int
        """

        self._size = QSize(w, h)
        self.setIconSize(self._size)
        self.setFixedSize(self._size)

    def update_icon(self):
        """
        Updates the icon for the current frame
        """

        if self._image_sequence.frames():
            frame_icon = self._image_sequence.current_icon()
            self.setIcon(frame_icon)

    def update_toolbar(self):
        """
        Updates the toolbar size depending on the number of actions
        """

        self._toolbar.setIconSize(QSize(16, 16))
        count = len(self.actions()) - 3
        width = 26 * count
        self._toolbar.setGeometry(0, 0, width, 25)
        x = self.rect().center().x() - (self._toolbar.width() * 0.5)
        y = self.height() - self._toolbar.height() - 12
        self._toolbar.setGeometry(x, y, self._toolbar.width(), self._toolbar.height())

    # =================================================================================================================
    # CALLBACKS
    # =================================================================================================================

    def _on_frame_changed(self, frame=None):
        """
        Internal callback function triggered when the image sequence changes frame
        :param frame: int or None
        """

        if not qtutils.is_control_modifier():
            frame_icon = self._image_sequence.current_icon()
            self.setIcon(frame_icon)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains history version widget for tpDcc-tools-datalibrary
"""

from __future__ import print_function, division, absolute_import

import os

from Qt.QtCore import Qt
from Qt.QtWidgets import QSizePolicy, QTreeWidgetItem, QMessageBox

from tpDcc.managers import resources
from tpDcc.libs.qt.core import qtutils, base
from tpDcc.libs.qt.widgets import layouts, buttons, treewidgets


class VersionHistoryTreeWidget(treewidgets.TreeWidget):

    HEADER_LABELS = ['Version', 'Comment', 'User', 'Date', 'Commit']

    def __init__(self, parent=None):
        super(VersionHistoryTreeWidget, self).__init__(parent=parent)

        self.setHeaderLabels(self.HEADER_LABELS)

        if qtutils.is_pyside() or qtutils.is_pyside2():
            self.sortByColumn(0, Qt.SortOrder.DescendingOrder)

        self.setColumnWidth(0, 70)
        self.setColumnWidth(1, 200)
        self.setColumnWidth(2, 70)
        self.setColumnWidth(3, 70)
        self.setColumnWidth(4, 70)
        self._padding = 1

    def update_versions_from_commits_data(self, commits_data, current_commit_data=None):
        self.clear()
        if not commits_data:
            return

        # https://stackoverflow.com/questions/42896141/how-to-lazily-iterate-on-reverse-order-of-keys-in-ordereddict/42896182#42896182
        for i, (commit, commit_data) in enumerate(
                ((commit, commits_data[commit]) for commit in reversed(commits_data))):
            item = QTreeWidgetItem()
            item.setText(0, str(i + 1))
            item.setText(1, commit_data.get('message', ''))
            item.setText(2, commit_data.get('author', ''))
            item.setText(3, commit_data.get('date', ''))
            item.setText(4, commit)
            self.addTopLevelItem(item)

            if current_commit_data:
                for current_commit in list(current_commit_data.keys()):
                    if current_commit == commit:
                        item.setBackgroundColor(0, Qt.green)


class VersionHistoryWidget(base.DirectoryWidget):
    def __init__(self, parent=None):
        super(VersionHistoryWidget, self).__init__(parent=parent)

        self._repository_path = None
        self._repository_type_class = None

    def ui(self):
        super(VersionHistoryWidget, self).ui()

        self.setSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.MinimumExpanding)
        self._btn_layout = layouts.HorizontalLayout()
        self._sync_btn = buttons.BaseButton('Sync', parent=self)
        self._sync_btn.setIcon(resources.icon('sync'))
        self._sync_btn.setMaximumWidth(100)
        self._sync_btn.setEnabled(False)
        self._btn_layout.addWidget(self._sync_btn)
        self._versions_tree = VersionHistoryTreeWidget(parent=self)
        self.main_layout.addWidget(self._versions_tree)
        self.main_layout.addLayout(self._btn_layout)

    def setup_signals(self):
        self._sync_btn.clicked.connect(self.sync_version)
        self._versions_tree.itemSelectionChanged.connect(self._on_update_selection)

    def set_directory(self, directory):
        super(VersionHistoryWidget, self).set_directory(directory)
        return self.refresh_versions()

    def set_repository_path(self, repository_path, refresh=False):
        self._repository_path = repository_path
        if refresh:
            return self.refresh_versions()

        return True

    def set_version_control_class(self, version_control_class, refresh=False):
        self._repository_type_class = version_control_class
        if refresh:
            return self.refresh_versions()

        return True

    def refresh_versions(self):
        self._versions_tree.clear()
        if not self._repository_type_class or not self._repository_path or not self.directory or not os.path.isdir(
                self._repository_path) or not os.path.isfile(self.directory):
            return False

        if not self._repository_type_class.is_valid_repository_directory(self._repository_path):
            return False

        commits_data = self._repository_type_class.get_commits_that_modified_a_file(
            self._repository_path, self.directory)
        if not commits_data:
            return False

        self._versions_tree.update_versions_from_commits_data(commits_data)

        return True

    def sync_version(self):
        """
        Opens selected version
        Override functionality for specific data
        """

        if not self._repository_type_class:
            return

        items = self._versions_tree.selectedItems()
        if not items:
            return
        item = items[0]

        item_commit = item.text(4)
        if not item_commit:
            return

        res = qtutils.show_question(
            self, 'Syncing File', 'Are you sure you want to synchronize this file?.'
                                  '\nCurrent local file will be overwriten!')
        if res != QMessageBox.Yes:
            return

        self._repository_type_class.sync_file(self._repository_path, self.directory, item_commit)

        # TODO: Dependencies should be checked (by checkding DB) and also, should be updated to commit
        # if that dependency was edited in the commit

    # =================================================================================================================
    # CALLBACKS
    # =================================================================================================================

    def _on_update_selection(self):
        items = self._versions_tree.selectedItems()
        if not items:
            self._sync_btn.setEnabled(False)
        else:
            self._sync_btn.setEnabled(True)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains data library libraries widget implementation
"""

from __future__ import print_function, division, absolute_import

from functools import partial

from Qt.QtWidgets import QMenu, QAction

from tpDcc.managers import resources
# from tpDcc.libs.datalibrary.core import utils


class LibrariesMenu(QMenu, object):
    def __init__(self, settings_path=None, library_window=None):
        super(LibrariesMenu, self).__init__(library_window)

        self._library_window = library_window
        self._settings_path = settings_path
        self.setTitle('Libraries')
        self.setIcon(resources.icon('books'))

        self.refresh()

    def settings_path(self):
        return self._settings_path

    def set_settings_path(self, settings_path):
        self._settings_path = settings_path
        self.refresh()

    def refresh(self):
        self.clear()

        # libraries = utils.read_settings(self._settings_path)
        # default = utils.default_library(self._settings_path)
        #
        # for name in libraries:
        #     library = libraries[name]
        #     path = library.get('path', '')
        #     kwargs = library.get('kwargs', dict())
        #     enabled = True
        #     if self._library_window:
        #         enabled = name != self._library_window.name()
        #     text = name
        #     if name == default and name.lower() != 'default':
        #         text = name + ' (default)'
        #
        #     action = QAction(text, self)
        #     action.setEnabled(enabled)
        #     action_callback = partial(self._on_show_library, name, path, **kwargs)
        #     action.triggered.connect(action_callback)
        #     self.addAction(action)

    def _on_show_library(self, name, path, **kwargs):
        """
        Internal callback function that shows the library window which has given name and path
        :param name: str
        :param path: str
        :param kwargs: dict
        """

        raise NotImplementedError()
